'use strict';

var any = require('any');
var iterator = require('make-iterator');
var cloneDeep = require('clone-deep');

/**
 * Returns `true` if `value` exists in the given string, array
 * or object. See [any] for documentation.
 *
 * @param {*} `value`
 * @param {*} `target`
 * @param {Object} `options`
 * @api public
 */

exports.any = function any_(value, target) {
  try {
    return any.apply(any, arguments);
  } catch (err) {}
  return '';
};

exports.any = function pick(o, props) {
  var keys = typeof props === 'string'
    ? [].slice.call(arguments, 1)
    : props;

  var res = {};
  var i = 0, key;

  while (key = keys[i++]) {
    res[key] = o[key];
  }
  return res;
};

function pluck(o, key){
  return map(o, prop(key));
}

function prop(key){
  return function(o){
    return o[key];
  };
}

function omit(o, keys) {
  var res = {};
  for (var key in o) {
    if (o.hasOwnProperty(key) && !contains(keys, key)) {
      res[key] = o[key];
    }
  }
  return res;
}

function get(o, prop) {
  var segs = prop.split('.');
  var last = segs.pop();

  while (prop = segs.shift()) {
    o = o[prop];
    if (o == null) {
      return;
    }
  }
  return o[last];
}

function contains(o, val) {
  return some(o, function (ele) {
    return (ele === val);
  });
}

function some(o, cb, thisArg) {
  cb = iterator(cb, thisArg);
  var res = false;

  forOwn(o, function (val, key) {
    if (cb(val, key, o)) {
      res = true;
      return false;
    }
  });
  return res;
}

function filter(o, cb, thisArg) {
  cb = iterator(cb, thisArg);
  var res = {};

  forOwn(o, function (val, key, obj) {
    if (cb(val, key, obj)) {
      res[key] = val;
    }
  });
  return res;
}

function map(o, cb, thisArg) {
  cb = iterator(cb, thisArg);
  var res = {};

  forOwn(o, function (val, key, o) {
    res[key] = cb(val, key, o);
  });
  return res;
}

function extend(o, objects) {
  var len = arguments.length;
  var i = 0;
  var obj;

  while (++i < len) {
    obj = arguments[i];
    if (obj != null) {
      forOwn(obj, function(val, key) {
        this[key] = val;
      }, o);
    }
  }
  return o;
}

function forOwn(o, fn, thisArg) {
  forIn(o, function (val, key) {
    if (hasOwn(o, key)) {
      return fn.call(thisArg, o[key], key, o);
    }
  });
}

function forIn(o, fn, thisArg) {
  var key, i = 0;

  for (key in o) {
    if (fn.call(thisArg, o[key], key, o) === false) {
      break;
    }
  }
}

/**
 * Return a function that will execute in the given context,
 * optionally adding any additional supplied parameters to
 * the beginning of the arguments collection.
 *
 * @param  {Function} fn
 * @param  {Object} thisArg
 * @param  {Object} args
 * @return {Object}
 */

function bind(fn, thisArg, args) {
  var arr = [].slice.call(arguments, 2);
  return function() {
    return fn.apply(thisArg, arr.concat([].slice.call(arguments)));
  };
}

function hasOwn(o, key){
 return Object.prototype.hasOwnProperty.call(o, key);
}
